// PG_DITCtrlEntry.cpp: implementation of the PG_DITCtrlEntry class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PCPages32.h"
#include "PG_DITCtrlEntry.h"
#include "PG_DITCtrl.h"
#include "QDN.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PG_DITCtrlEntry::PG_DITCtrlEntry
	( QRDN *pRDN, QEntry *pEntry, PG_DITCtrlEntry *pParentEntry, PG_DITCtrl *pTreeWnd )
{
	extern CPCPages32App theApp;

  m_RDN = *pRDN;
	m_pParentEntry = pParentEntry;
  m_pTreeWnd = pTreeWnd;
  m_bHasChildren = m_bExpanded = FALSE;

	//
	// Get the RDN string
	CString rdnStr;
	m_RDN.GetFriendlyRDNString( rdnStr );
	strcpy( m_nameBuf, rdnStr );

  //
  // Add the new item to the tree
  TV_INSERTSTRUCT tvi;
  tvi.hParent =
		( pParentEntry->m_hTreeItem == NULL?
			TVI_ROOT : pParentEntry->m_hTreeItem );

  tvi.hInsertAfter = TVI_FIRST;
  tvi.item.mask = TVIF_TEXT|TVIF_PARAM|TVIF_CHILDREN;
  tvi.item.pszText = m_nameBuf;

	int nImageID;
	if ( theApp.Config().IconMap().GetIconID( pEntry, nImageID ) == TRUE )
		{
			tvi.item.mask |= TVIF_IMAGE|TVIF_SELECTEDIMAGE;
			tvi.item.iImage = tvi.item.iSelectedImage = nImageID;
		}

  tvi.item.cchTextMax = 255;
  tvi.item.cChildren = I_CHILDRENCALLBACK;

  tvi.item.lParam = (LPARAM) this;
  m_hTreeItem = pTreeWnd->InsertItem( &tvi );
}

//
// This constructor used to create the dummy root entry.
PG_DITCtrlEntry::PG_DITCtrlEntry( PG_DITCtrl *pTreeWnd )
{
	m_pParentEntry = NULL;
	m_hTreeItem = NULL;
  m_pTreeWnd = pTreeWnd;
  m_bHasChildren = m_bExpanded = FALSE;
}

PG_DITCtrlEntry::~PG_DITCtrlEntry()
{
	POSITION pos = m_childList.GetHeadPosition();
	while ( pos != NULL )
		{
			PG_DITCtrlEntry * pEntry = m_childList.GetNext( pos );
			delete pEntry;
		}
}

//////////////////////////////////////////////////////////////////////
// Implementation
//////////////////////////////////////////////////////////////////////

BOOL PG_DITCtrlEntry::AddChild( QEntry *pEntry, QRDN *pRDN, BOOL bShow )
{
	//
	// XXX Add object class to PG_DITCtrlEntry constructor in PG_DITCtrl::AddChild
	PG_DITCtrlEntry *pChild = 
		new PG_DITCtrlEntry( pRDN, pEntry, this, m_pTreeWnd );

	m_childList.AddHead( pChild );

	if ( bShow )
		m_pTreeWnd->EnsureVisible( pChild->m_hTreeItem );

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// PG_DITCtrlEntry::GetChild
// Get the named child of a given entry present in the tree control.
// Return FALSE if the child is not present.
BOOL PG_DITCtrlEntry::GetChild( QRDN &rRDN, PG_DITCtrlEntry **pChildPtr )
{
	POSITION pos = m_childList.GetHeadPosition();
	while ( pos != NULL )
		{
			PG_DITCtrlEntry *pChild = m_childList.GetNext( pos );
			if ( pChild->RDN() == rRDN )
				{
					*pChildPtr = pChild;
					return TRUE;
				}
		}

	return FALSE;
}

void PG_DITCtrlEntry::SetChildState()
{
  //
  // If no children tag the tree item as having no children...
  if (m_bHasChildren == 0 && m_bExpanded == TRUE)
    {
      TV_ITEM item;
      item.hItem = m_hTreeItem;
      item.mask = TVIF_CHILDREN;
      item.cChildren = 0;
      m_pTreeWnd->SetItem( &item );
    }
}

//
// Display children, and also send a browse request to look for more.
void PG_DITCtrlEntry::Expand()
{
  m_pTreeWnd->Expand( m_hTreeItem, TVE_EXPAND );
}

//
// Set the collapse/expand button (+ or -) to indicate whether a browse
// operation has been performed for this entry.
void PG_DITCtrlEntry::SetExpandedState()
{
  //
  // Keep showing the "+" button if the entry hasn't been browsed yet
  if (m_bExpanded == FALSE)
		{
			m_pTreeWnd->SetItemState
				( m_hTreeItem, TVIS_EXPANDPARTIAL, TVIS_EXPANDPARTIAL );
		}
}

//
// Display children, but only those that are already there.
// This basically means that the interface won't attempt to 
// look for any more children by sending a browse request.
void PG_DITCtrlEntry::ExpandPartial()
{
  BOOL bTmp = m_bExpanded;
  m_bExpanded = TRUE;

  m_pTreeWnd->Expand( m_hTreeItem, TVE_EXPAND|TVE_EXPANDPARTIAL );
  m_bExpanded = bTmp;
}





