// PG_EntryListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "PCPages32.h"
#include "PG_EntryListCtrl.h"
#include "QEntry.h"
#include "MainFrm.h"
#include "PG_RHSTabCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl

PG_EntryListCtrl::PG_EntryListCtrl()
{
	m_pEntryReader = NULL;
	m_nMenuID = -1;

	//
	// Multiple selection disabled by default
	m_bMultiSel = FALSE;
}

PG_EntryListCtrl::~PG_EntryListCtrl()
{
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::Create
void PG_EntryListCtrl::Create( CWnd *pParentWnd, RECT &rect, UINT nId )
{
	extern CPCPages32App theApp;

	DWORD dwStyle = LVS_REPORT|WS_BORDER;
	if ( m_bMultiSel == FALSE )
		dwStyle |= LVS_SINGLESEL;

  CListCtrl::Create( dwStyle, rect, pParentWnd, nId );
	SetExtendedStyle
		( GetExtendedStyle() | LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP );

 	//
	// Load images
	m_imageList.Create( IDB_ENTRY_ICONS, 19, 0, RGB( 0xff, 0xff, 0xff ) );
	SetImageList( &m_imageList, LVSIL_SMALL );

	//
	// Load configuration from registry
	if ( m_elConfig.ReadConfiguration() == FALSE )
		theApp.ErrorBox( PGS_REGINIT_FAILED );

	//
	// Create the columns according to loaded configuration information
	QStringList colList;
	m_elConfig.GetColumnList( colList );

  LPCTSTR str;
  UINT i = 0;

  POSITION pos = colList.GetHeadPosition();
  while ( pos != NULL )
    {
      str = ( LPCTSTR ) colList.GetNext( pos );
      InsertColumn( i, str, LVCFMT_LEFT, 150, i );
      i++;
    }
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::AddEntry
// Add an entry to the list. This takes control of the supplied entry.
void PG_EntryListCtrl::AddEntry( QEntry *pNewEntry )
{
	LV_ITEM lvi;

	//
	// Make sure that the entry isn't already in the list
	POSITION pos = m_entryList.GetHeadPosition();
	while ( pos != NULL )
		{
			QEntry *pEntry = m_entryList.GetNext( pos );
			if ( pEntry->DN() == pNewEntry->DN() )
				return;
		}

  //
  // Set base entry attributes (name and entry data pointer)
  lvi.iItem = 0;
  lvi.lParam = reinterpret_cast<long>( pNewEntry );

	//
	// Get string entry for first column
	CString strVal;
	m_elConfig.GetColumnValue( pNewEntry, 0, strVal );
	lvi.pszText = strdup( strVal );

  //
  // Add the base entry (i.e. column 0) 
  lvi.mask = LVIF_TEXT|LVIF_PARAM;
  lvi.iSubItem = 0;
  int nItemID = InsertItem( &lvi );
	free( lvi.pszText );

	//
	// Add remaining columns
	QStringList colList;
	UINT nColCount = m_elConfig.GetColumnList( colList );
	for ( UINT nColID = 1; nColID < nColCount; nColID++ )
		{
			m_elConfig.GetColumnValue( pNewEntry, nColID, strVal );
			lvi.pszText = strdup( strVal );

			lvi.mask = LVIF_TEXT;
			lvi.iSubItem = nColID;

			SetItem( &lvi );
			free( lvi.pszText );
		}

	//
	// Add entry to list
	m_entryList.AddHead( pNewEntry );
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::AddEntryList
// Add a number of entries to the list. This takes control of the
// supplied entries.
void PG_EntryListCtrl::AddEntryList( PtrList<QEntry> &rEntryList )
{
	POSITION pos = rEntryList.GetHeadPosition();
	while ( pos != NULL )
		{
			QEntry *pEntry = rEntryList.GetNext( pos );
			AddEntry( pEntry );
		}
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::RemoveEntry
void PG_EntryListCtrl::RemoveEntry( QDN &dn )
{
	// XXX PG_EntryListCtrl::RemoveEntry unimplemented
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::RemoveAllEntries
void PG_EntryListCtrl::RemoveAllEntries()
{
	if ( m_entryList.IsEmpty() == TRUE )
		return;

	DeleteAllItems();

	POSITION pos = m_entryList.GetHeadPosition();
	while ( pos != NULL )
		{
			QEntry *pEntry = m_entryList.GetNext( pos );
			delete pEntry;
		}

	m_entryList.RemoveAll();
}

BEGIN_MESSAGE_MAP(PG_EntryListCtrl, CListCtrl)
	//{{AFX_MSG_MAP(PG_EntryListCtrl)
	ON_WM_DESTROY()
	ON_NOTIFY_REFLECT(HDN_ITEMDBLCLICK, OnItemdblclick)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclk)
	ON_NOTIFY_REFLECT(LVN_BEGINDRAG, OnBegindrag)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_ENTRY_LIST_COLLECT, OnCollectEntry)
	ON_COMMAND(ID_ENTRY_LIST_MAIL, OnMailTo)
	ON_COMMAND(ID_ENTRY_LIST_VIEW, OnViewEntry)
	ON_COMMAND(ID_ENTRY_CONTAINER_DELETE, OnRemoveEntry)
	ON_WM_MENUSELECT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl message handlers

void PG_EntryListCtrl::OnDestroy() 
{
	RemoveAllEntries();
	CListCtrl::OnDestroy();
}

void PG_EntryListCtrl::OnItemdblclick( NMHDR* pNMHDR, LRESULT* pResult ) 
{
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::OnDblClick
// Display the selected entry.
void PG_EntryListCtrl::OnDblclk( NMHDR* pNMHDR, LRESULT* pResult ) 
{
	if ( m_pEntryReader != NULL &&
				m_pEntryReader->Invoked() == TRUE &&
				m_pEntryReader->Finished() == FALSE )
		{
			//
			// Already reading an entry!
			*pResult = 0;
			return;
		}

	if ( m_pEntryReader != NULL )
		{
			delete m_pEntryReader;
			m_pEntryReader = NULL;
		}

	//
	// Get the selected list item
	int item = GetNextItem( -1, LVNI_SELECTED );
	DisplayEntry( item );
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::OnMailTo
void PG_EntryListCtrl::DisplayEntry( int nItemID )
{
  LV_ITEM lvi;
  if ( nItemID < 0)
    {
      return;
    }

  //
  // Get entry pointer of the item
  lvi.iItem = nItemID;
  lvi.mask = LVIF_PARAM;

  GetItem( &lvi );
  QEntry *pEntry = reinterpret_cast<QEntry *>( lvi.lParam );

  //
  // Request a read and display operation
	extern CPCPages32App theApp;
	theApp.SetMessageText( "" );
	theApp.ClearErrors();

	m_pEntryReader = 
		new QReadEntry
			( theApp.DS(), theApp.DS()->GenRequestID(), &m_readHandler, pEntry->DN() );

	QError *pErr = NULL;
	if ( m_pEntryReader->Invoke( pErr ) == FALSE )
		{
#ifdef _DEBUG
			CString errStr;
			pErr->Print( errStr );

			TRACE
				( " *** PG_EntryListCtrl::OnDblclk()\
					\nFailed to read an entry from the directory, error message is:\
					\n%s\n\n",
					errStr );
#endif
			theApp.ErrorBox( PGS_DSREAD_FAILED );

			if ( pErr != NULL )
				theApp.AddError( pErr );
		}
}

void PG_EntryListCtrl::OnBegindrag( NMHDR* pNMHDR, LRESULT* pResult ) 
{
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::SetContextMenuResID
// Set the resource ID of the context menu.
void PG_EntryListCtrl::SetContextMenuResID( int nResID )
{
	m_nMenuID = nResID;
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::OnContextMenu
// Pop up the context menu.
void PG_EntryListCtrl::OnContextMenu( CWnd* pWnd, CPoint point )
{
	if ( m_nMenuID == -1 )
		return;

	//
	// Get the point in window relative co-ordinates
	CPoint local = point;
	ScreenToClient( &local );

	//
	// Select the entry that the cursor is over
	int nID = HitTest( local );
	if ( nID < 0 )
		return;

	//
	// If the cursor is NOT over a selected entry then modify the
	// selection to reflect this.
	int nSelNum = 0;
	BOOL bSelected = FALSE;
	POSITION pos = GetFirstSelectedItemPosition();
	while ( pos != NULL )
		{
			int nSelID = GetNextSelectedItem( pos );
			if ( nSelID == nID )
				bSelected = TRUE;

			nSelNum++;
		}

	//
	// Unselect all entries and then select the one
	// that the cursor is over
	if ( bSelected == FALSE )
		{
			pos = GetFirstSelectedItemPosition();
			while ( pos != NULL )
				{
					int nSelID = GetNextSelectedItem( pos );
					SetItemState( nSelID, LVIS_SELECTED, 0 );
				}

			SetItemState( nID, LVIS_SELECTED, LVIS_SELECTED );
			nSelNum = 1;
		}
	
	//
	// Load the menu resource
	CMenu menu;
	if ( menu.LoadMenu( m_nMenuID ) == TRUE )
		{
			CMenu* pPopupMenu = menu.GetSubMenu( 0 );
			if ( pPopupMenu != NULL )
				{
					//
					// Enable/disable menu items as appropriate
					if ( nSelNum > 1 )
						{
							pPopupMenu->EnableMenuItem
								( ID_ENTRY_LIST_VIEW, MF_GRAYED|MF_BYCOMMAND );
						}

					//
					// Display and track it
					pPopupMenu->TrackPopupMenu
						( TPM_LEFTALIGN|TPM_RIGHTBUTTON, point.x, point.y, this );
				}

			menu.DestroyMenu();
		}
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::OnCollectEntry
// Add the selected entry to the collection.
void PG_EntryListCtrl::OnCollectEntry() 
{
	extern CPCPages32App theApp;

	POSITION pos = GetFirstSelectedItemPosition();
	while ( pos != NULL )
		{
			int nID = GetNextSelectedItem( pos );
		  if ( nID < 0 )
			  continue;

			//
			// Get entry pointer of the item
			LV_ITEM lvi;
			lvi.iItem = nID;
			lvi.mask = LVIF_PARAM;

			GetItem( &lvi );
			QEntry *pEntry = reinterpret_cast<QEntry *>( lvi.lParam );

			//
			// Add the entry to the collector control
			QEntry *pCopyEntry = new QEntry( pEntry );
			theApp.EntryListContainer().AddEntry( pCopyEntry );

			//
			// Display the "zoom" indicator
			CRect toRect, fromRect;
			GetItemRect( nID, fromRect, LVIR_BOUNDS );

			PG_RHSTabCtrl *pRhsTab = theApp.MainFrame()->m_pRhsTabCtrl;
			pRhsTab->GetItemRect( pRhsTab->EntryContainerID(), toRect );

			theApp.Flash( pRhsTab, fromRect, toRect );
		}
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::OnMailTo
// Add the selected entry to the collection.
void PG_EntryListCtrl::OnMailTo() 
{
	extern CPCPages32App theApp;
	QStringList addrList;
	int addrNum = 0, noAddrNum = 0;
	
	//
	// Get one e-mail address from each selected entry
	POSITION pos = GetFirstSelectedItemPosition();
	while ( pos != NULL )
		{
			int nID = GetNextSelectedItem( pos );

			//
			// Get the entry
			LV_ITEM lvi;
			lvi.iItem = nID;
			lvi.mask = LVIF_PARAM;

			GetItem( &lvi );
			QEntry *pEntry = reinterpret_cast<QEntry *>( lvi.lParam );

			addrNum++;

			//
			// Get the e-mail address attribute
			QAttr *pMailAttr =
				pEntry->QueryGetAttr
					( theApp.Oidtable()->str2at( CString( "rfc822mailbox" ) ) );

			if ( pMailAttr == NULL )
				{
					noAddrNum++;
					continue;
				}

			QStringList valList;
			pMailAttr->PrintValues( valList );

			if ( valList.GetCount() == 0 )
				{
					noAddrNum++;
					continue;
				}

			//
			// Add the e-mail address to the list
			addrList.AddTail( valList.GetHead() );
		}

	//
	// Check for errors and for entries that don't have e-mail
	// addresses
	if ( noAddrNum > 0 )
		{
			if ( addrNum == noAddrNum )
				{
					CString msgStr;
					msgStr.Format( PGS_ADDRESSES_MISSING, noAddrNum, addrNum );

					if ( AfxMessageBox( msgStr, MB_YESNO|MB_ICONSTOP ) == IDNO )
						return;
				}
		}

	//
	// Build "mailto" string
	CString mailStr = "mailto:";
	BOOL bFirst = TRUE;

	pos = addrList.GetHeadPosition();
	while ( pos != NULL )
		{
			CString addrStr = addrList.GetNext( pos );
			if ( bFirst == FALSE )
				mailStr += ";";

			mailStr += addrStr;
			bFirst = FALSE;
		}

	//
	// Execute the mail command
	::ShellExecute( ::GetDesktopWindow(), "open", mailStr, NULL, "", SW_SHOW);
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::OnViewEntry
void PG_EntryListCtrl::OnViewEntry() 
{
	int nID = GetNextItem( -1, LVNI_SELECTED );
	DisplayEntry( nID );
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::OnRemoveEntry
// Remove selected entries from the list.
void PG_EntryListCtrl::OnRemoveEntry() 
{
	POSITION pos = GetFirstSelectedItemPosition();
	while ( pos != NULL )
		{
			int nID = GetNextSelectedItem( pos );

			//
			// Get the entry
			LV_ITEM lvi;
			lvi.iItem = nID;
			lvi.mask = LVIF_PARAM;

			GetItem( &lvi );
			QEntry *pEntry = reinterpret_cast<QEntry *>( lvi.lParam );

			POSITION entryPos = m_entryList.Find( pEntry );
			m_entryList.RemoveAt( entryPos );

			delete pEntry;
		}

	//
	// Delete the list item
	while ( ( pos = GetFirstSelectedItemPosition() ) != NULL )
		{
			int nID = GetNextSelectedItem( pos );
			DeleteItem( nID );
		}
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::OnMenuSelect
void PG_EntryListCtrl::OnMenuSelect(UINT nItemID, UINT nFlags, HMENU hSysMenu)
{
	extern CPCPages32App theApp;
	CString str;

	if ( nFlags & ( MF_POPUP | MF_SEPARATOR | MF_SYSMENU | MF_GRAYED ) ||
				nFlags == 0xffff )
		str = "";
	else
		str.LoadString( nItemID );

#ifdef _DEBUG
	TRACE( "Selected menu item: ID %d, Flags %d\n", nItemID, nFlags );
#endif

	theApp.SetMessageText( str );
}
