// PG_EntryViewConfig.cpp: implementation of the PG_EntryViewConfig,
// PG_EntryViewClass and PG_EntryViewAttr classes.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PCPages32.h"
#include "PG_EntryViewConfig.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// PG_EntryViewClass Construction/Destruction
//////////////////////////////////////////////////////////////////////

PG_EntryViewClass::PG_EntryViewClass
	( QOC *pOC, PtrList<PG_EntryViewAttr> &rEVAList )
{
	POSITION pos = rEVAList.GetHeadPosition();
	while ( pos != NULL )
		m_EVAList.AddTail( rEVAList.GetNext( pos ) );

	m_pOC = pOC;
}

PG_EntryViewClass::~PG_EntryViewClass()
{
	POSITION pos = m_EVAList.GetHeadPosition();
	while ( pos != NULL )
		delete m_EVAList.GetNext( pos );
}

//////////////////////////////////////////////////////////////////////
// PG_EntryViewClass::QueryGetEVAList
//
// Retrieve m_EVAList if the given object classes matches this
// entry display class.
BOOL PG_EntryViewClass::QueryGetEVAList
	( QOC *pOC, PtrList<PG_EntryViewAttr> &rEVAList )
{
	if ( pOC != m_pOC )
		return FALSE;

	rEVAList.RemoveAll();

	POSITION pos = m_EVAList.GetHeadPosition();
	while ( pos != NULL )
		rEVAList.AddTail( m_EVAList.GetNext( pos ) );

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// PG_EntryViewConfig Construction/Destruction
//////////////////////////////////////////////////////////////////////

PG_EntryViewConfig::PG_EntryViewConfig()
{

}

PG_EntryViewConfig::~PG_EntryViewConfig()
{
	POSITION pos = m_viewClassList.GetHeadPosition();
	while ( pos != NULL )
		delete m_viewClassList.GetNext( pos );
}

//////////////////////////////////////////////////////////////////////
// PG_EntryViewConfig::Load
//
// Load the configuration from the registry.
BOOL PG_EntryViewConfig::Load()
{
	extern CPCPages32App theApp;

	HKEY hKey = theApp.OpenAppKey( "Entry Display" );
	if ( hKey == NULL )
		{
			theApp.ErrorBox( PGS_REGINIT_FAILED );
			return FALSE;
		}

	//
	// Get key/value information
	DWORD dwNumValues;
	DWORD dwMaxValueLen;
	DWORD dwMaxValueNameLen;
	
	LONG ok = RegQueryInfoKey
		(
			hKey,                // handle to key to query
			NULL,           // address of buffer for class string
			NULL,        // address of size of class string buffer
			NULL,       // reserved
			NULL,       // address of buffer for number of subkeys
			NULL,  // address of buffer for longest subkey name length
			NULL,  // address of buffer for longest class string length
			&dwNumValues,        // address of buffer for number of value entries
			&dwMaxValueNameLen,  // address of buffer for longest value name length
			&dwMaxValueLen,  // address of buffer for longest value data length
			NULL,           // address of buffer for security descriptor length
			NULL						// address of buffer for last write time
		);

	//
	// Create a buffer for loading
	BYTE *pBuf = new BYTE[ dwMaxValueLen ];
	DWORD dwBufLen = dwMaxValueLen;
	
	//
	// Read the set of entry classes (an object class list)
	if ( theApp.QueryValue
				( hKey, "Object Classes", REG_SZ, pBuf, &dwBufLen ) == FALSE )
		{
			theApp.ErrorBox( PGS_REGINIT_FAILED );
			delete pBuf;
			return FALSE;
		}

	//
	// Got the string now convert to a string list (semi-colon seperated)
	CString ocStr( reinterpret_cast<char *>( pBuf ) );
	QStringList ocStrList;
	ocStrList.SetSplitString( ocStr, ';' );

	//
	// Read the attribute list for each of the listed object classes,
	// then create and add a new PG_EntryViewClass.
	POSITION pos = ocStrList.GetHeadPosition();
	while ( pos != NULL )
		{
			QAttrT *pAT;
			QOC *pOC;
			DWORD pref;

			pAT = NULL;
			pOC = NULL;
			pref = PG_EVAPREF_NONE;

			dwBufLen = dwMaxValueLen;
			PtrList<PG_EntryViewAttr> eVAList;

			//
			// Read the list of entry view attribute (EVA) specifications
			// for each object class. The EVA string consists of a set of
			// colon seperated values - the attribute type followed by a
			// set of optional display preferences:
			//
			// "rdn" - Retrieve the attribute from the RDN (if present there)
			// "title" - Display the attribute using a "title" style (i.e.
			//					 in a large font and without the attribute type
			//					 displayed)
			CString ocStr = ocStrList.GetNext( pos );

			pOC = theApp.Oidtable()->str2oc( ocStr );
			if ( pOC == NULL )
				{
#ifdef _DEBUG
					TRACE
						( "PG_EntryViewConfig::Load()\
							\nInvalid object class found in registry value:\
							\nRegKey: HKCU/Software/Brunel University/PC-Pages/Entry Display\
							\nRegValue: Object Classes\
							\nInvalid Value: %s\n\n",
							ocStr );
#endif
					continue;
				}

			char *pTmpStr = strdup( ocStr );
			if ( theApp.QueryValue
						( hKey, pTmpStr, REG_SZ, pBuf, &dwBufLen ) == TRUE )
				{
					free( pTmpStr );

					//
					// Split the EVA string into a sequence of the attribute type
					// and view preferences, e.g. cn:rdn:title
					CString eVAStr( reinterpret_cast<char *>( pBuf ) );
					QStringList eVAStrList;
					eVAStrList.SetSplitString( eVAStr, ';' );

					//
					// Get the attribute type
					POSITION eVAPos = eVAStrList.GetHeadPosition();
					while ( eVAPos != NULL )
						{
							eVAStr = eVAStrList.GetNext( eVAPos );
							QStringList eVAWordList;

							eVAWordList.SetSplitString( eVAStr, ':' );
							POSITION wordPos = eVAWordList.GetHeadPosition();
							CString eVAWord = eVAWordList.GetNext( wordPos );

							pAT =	theApp.Oidtable()->str2at( eVAWord );
							if ( pAT == NULL )
								{
#ifdef _DEBUG
									TRACE
										( "Invalid attribute type (%s) found below PC-Pages registry key \"Entry Display\".\n\n",
											eVAWord );
#endif
									continue;
								}

							//
							// Get the display preferences
							pref = PG_EVAPREF_NONE;
							while ( wordPos != NULL )
								{
									eVAWord = eVAWordList.GetNext( wordPos );
									if ( eVAWord == "rdn" )
										{
											pref |= PG_EVAPREF_RDN;
										}
									else if ( eVAWord == "title" )
										{
											pref |= PG_EVAPREF_TITLE;
										}
									else if ( eVAWord == "shellex" )
										{
											pref |= PG_EVAPREF_SHELLEX;
										}
								}

							//
							// Read a complete EVA. Now create it and it to the list
							eVAList.AddTail( new PG_EntryViewAttr( pAT, pref ) );
						}
				}

			if ( pOC != NULL && eVAList.GetCount() > 0 )
				{
					PG_EntryViewClass *pEVClass = new PG_EntryViewClass( pOC, eVAList );
					m_viewClassList.AddTail( pEVClass );
				}
		}

	//
	// All done
	delete pBuf;
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// PG_EntryViewConfig::QueryGetEVAList
//
// Get the attribute display list configured for the given entry.
BOOL PG_EntryViewConfig::QueryGetEVAList
	( QEntry *pEntry, PtrList<PG_EntryViewAttr> &rEVAList )
{
	extern CPCPages32App theApp;
	PtrList<QValue> valList;

	QAttr *pAttr = pEntry->ObjectClass();
	pAttr->GetValues( valList );

	POSITION pos = valList.GetHeadPosition();
	while ( pos != NULL )
		{
			CString ocValStr;
			QValue *pVal = valList.GetNext( pos );
			pVal->Print( ocValStr );

			QOC *pOC = theApp.Oidtable()->str2oc( ocValStr );
			if ( pOC != NULL && QueryGetEVAList( pOC, rEVAList) == TRUE )
				return TRUE;
		}

	return FALSE;
}

//////////////////////////////////////////////////////////////////////
// PG_EntryViewConfig::QueryGetEVAList
//
// Get the attribute display list configured for the given
// object class.
BOOL PG_EntryViewConfig::QueryGetEVAList
	( QOC *pOC, PtrList<PG_EntryViewAttr> &rEVAList )
{
	POSITION pos = m_viewClassList.GetHeadPosition();
	while ( pos != NULL )
		{
			PG_EntryViewClass * pViewClass = m_viewClassList.GetNext( pos );
			if ( pViewClass->QueryGetEVAList( pOC, rEVAList ) == TRUE )
				return TRUE;
		}

	return FALSE;
}

