// PG_EntryViewCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "PCPages32.h"
#include "PG_EntryViewCtrl.h"
#include "PG_EntryViewConfig.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define INITIAL_X_PADDING 12
#define	INITIAL_Y_PADDING 12
#define DEFAULT_X_PADDING 12
#define DEFAULT_Y_PADDING 0
#define VALUE_X_PADDING 24
#define VALUE_Y_PADDING 0
#define TITLE_X_PADDING 12
#define TITLE_Y_PADDING 10
#define ATTR_Y_PADDING 8


/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl

PG_EntryViewCtrl::PG_EntryViewCtrl()
{
	m_pEntry = NULL;
}

PG_EntryViewCtrl::~PG_EntryViewCtrl()
{
}

BEGIN_MESSAGE_MAP(PG_EntryViewCtrl, CWnd)
	//{{AFX_MSG_MAP(PG_EntryViewCtrl)
	ON_WM_PAINT()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::Create
// Create entry view control window.
void PG_EntryViewCtrl::Create( CWnd *pParentWnd, RECT &rect, UINT nId )
{
	CWnd::Create( NULL, "PG_EntryView", WS_CHILD|WS_BORDER, rect, pParentWnd, nId );
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::SetEntry
// Set the current entry.
void PG_EntryViewCtrl::SetEntry( QEntry *pEntry )
{
	ClearEntry();
	m_pEntry = pEntry;
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::ClearEntry
// Clear the current entry.
void PG_EntryViewCtrl::ClearEntry()
{
	if ( m_pEntry != NULL )
		{
			delete m_pEntry;
			m_pEntry = NULL;
		}

	POSITION pos = m_linkList.GetHeadPosition();
	while ( pos != NULL )
		{
			delete m_linkList.GetNext( pos );
		}

	m_linkList.RemoveAll();
	m_bFirstPaint = TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::CreateFonts
// Create the font structures necessary for drawing.
void PG_EntryViewCtrl::CreateFonts()
{
	if ( m_pValueFont == NULL )
		{
			m_pValueFont = new CFont();
			m_pValueFont->CreateFont
				(-14, 0,  // Width and height
				 0, 0,      // Escapement and orientation
				 0, 0, 0, 0, // Weight, italic, underline, strikeout
				 ANSI_CHARSET,
				 OUT_DEFAULT_PRECIS,
				 CLIP_DEFAULT_PRECIS,
				 DEFAULT_QUALITY,
				 VARIABLE_PITCH|FF_DONTCARE,
				 "Arial");
		}

	if ( m_pTypeFont == NULL )
		{
			m_pTypeFont = new CFont();
			m_pTypeFont->CreateFont
				(-14, 0,  // Width and height
				 0, 0,      // Escapement and orientation
				 FW_BOLD, 0, 0, 0, // Weight, italic, underline, strikeout
				 ANSI_CHARSET,
				 OUT_DEFAULT_PRECIS,
				 CLIP_DEFAULT_PRECIS,
				 DEFAULT_QUALITY,
				 VARIABLE_PITCH|FF_DONTCARE,
				 "Arial");
		}

	if ( m_pTitleFont == NULL )
		{
			m_pTitleFont = new CFont();
			m_pTitleFont->CreateFont
				(-32, 0,  // Width and height
				 0, 0,      // Escapement and orientation
				 FW_BOLD, 0, 0, 0, // Weight, italic, underline, strikeout
				 ANSI_CHARSET,
				 OUT_DEFAULT_PRECIS,
				 CLIP_DEFAULT_PRECIS,
				 DEFAULT_QUALITY,
				 VARIABLE_PITCH|FF_DONTCARE,
				 "Arial");
		}

	m_pOrigFont = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::DestroyFonts
// Destroy the font structures created temporarily for entry drawing.
void PG_EntryViewCtrl::DestroyFonts()
{
	if ( m_pDC == NULL )
		return;

	if ( m_pTypeFont != NULL )
		{
			delete m_pTypeFont;
			m_pTypeFont = NULL;
		}

	if ( m_pValueFont != NULL )
		{
			delete m_pValueFont;
			m_pValueFont = NULL;
		}

	if ( m_pTitleFont != NULL )
		{
			delete m_pTitleFont;
			m_pTitleFont = NULL;
		}
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::DrawEntry
// Draw the entry.
void PG_EntryViewCtrl::DrawEntry( CDC *pDC )
{
	extern CPCPages32App theApp;
	if ( m_pEntry == NULL )
		return;

	m_pDC = pDC;
	m_nNextDrawX = INITIAL_X_PADDING;
	m_nNextDrawY = INITIAL_Y_PADDING;

	CreateFonts();
	m_linkPos = m_linkList.GetHeadPosition();

	PtrList<PG_EntryViewAttr> eVAList;
	theApp.Config().EntryViewConfig().QueryGetEVAList( m_pEntry, eVAList );

	POSITION pos = eVAList.GetHeadPosition();
	while ( pos != NULL )
		{
			//
			// Retrieve the entry attribute associated with the
			// current entry view attribute template
			PG_EntryViewAttr *pEVA = eVAList.GetNext( pos );

			//
			// Display values
			if ( pEVA->ViewPreference() & PG_EVAPREF_RDN )
				DrawRDNAttr( pEVA );
			else
				DrawEntryAttr( pEVA );
		}

	DestroyFonts();
	m_pDC = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::DrawRDNAttr
// Draw the specified attribute of the RDN. If the attribute isn't present
// in the RDN try to get it from the body of the entry.
void PG_EntryViewCtrl::DrawRDNAttr( PG_EntryViewAttr *pEVA )
{
	QRDN rdn;
	m_pEntry->DN().GetRDN( rdn );
	
	POSITION pos = rdn.GetHeadRDNAttr();
	while ( pos != NULL )
		{
			QRDNAttr *pRDNAttr = rdn.GetNextRDNAttr( pos );
			if ( pRDNAttr->Type() == pEVA->Type() )
				{
					CString valStr;
					pRDNAttr->Value()->Print( valStr );

					if ( pEVA->ViewPreference() & PG_EVAPREF_TITLE )
						{
							DrawString( valStr, m_pTitleFont );
							m_nNextDrawY += TITLE_Y_PADDING;
						}
					else
						DrawString( valStr, m_pValueFont );

					m_nNextDrawY += ATTR_Y_PADDING;
					return;
				}
		}

	//
	// If fallen through to here then the attribute wasn't found
	// in the RDN. See if it's in the entry...
	DrawEntryAttr( pEVA, FALSE );
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::DrawEntryAttr
// Draw the specified attribute of the RDN. If the attribute isn't present
// in the RDN try to get it from the body of the entry.
void PG_EntryViewCtrl::DrawEntryAttr( PG_EntryViewAttr *pEVA, BOOL bShowType /* = TRUE */ )
{
	QAttr *pAttr = m_pEntry->QueryGetAttr( pEVA->Type() );
	if ( pAttr == NULL )
		return;

	PtrList<QValue> valList;
	pAttr->GetValues( valList );

	if ( valList.GetCount() == 0 )
		return;

	//
	// Draw the attribute type string
	if ( bShowType == TRUE )
		{
			CString typeStr = pAttr->Type()->FriendlyName();
			DrawString( typeStr, m_pTypeFont );
		}

	//
	// Draw each value string
	POSITION pos = valList.GetHeadPosition();
	while ( pos != NULL )
		{
			CString valStr;
			BOOL bDone = FALSE;
			
			QValue *pVal = valList.GetNext( pos );
			pVal->Print( valStr );

			if ( pEVA->ViewPreference() & PG_EVAPREF_SHELLEX )
				// Create as a hyperlink
				{
					CString dataStr;
					pVal->PrintRaw( dataStr );
					bDone = TRUE;
					CFont *pFont =
						( pEVA->ViewPreference() & PG_EVAPREF_TITLE?
							m_pTitleFont :
							m_pValueFont );

					//
					// Some attributes need a bit of special processing...
					if ( pAttr->Type()->HasName( CString( "mail" ) ) == TRUE )
						{
							dataStr = "mailto:" + dataStr;
						}
	
					//
					// Create the hyperlink window
					m_nNextDrawX = VALUE_X_PADDING;
					DrawLink( valStr, dataStr, pFont );
				}
			
			if ( pEVA->ViewPreference() & PG_EVAPREF_TITLE )
				{
					DrawString( valStr, m_pTitleFont );
					m_nNextDrawY += TITLE_Y_PADDING;
					bDone = TRUE;
				}
			
			if ( bDone == FALSE )
				{
					m_nNextDrawX = VALUE_X_PADDING;
					DrawString( valStr, m_pValueFont );
				}
		}

	m_nNextDrawY += ATTR_Y_PADDING;
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::DrawLink
// Draw the given string as a hyper link.
void PG_EntryViewCtrl::DrawLink
	( CString &rDrawStr, CString &rLinkStr, CFont *pFont, int nXExtra, int nYExtra )
{
	//
	// Create the hyperlink control if painting for the first time
	// otherwise get the next one from the list
	CHyperlink *pLink;
	if ( m_bFirstPaint == TRUE )
		{
			pLink = new CHyperlink();
			m_linkList.AddTail( pLink );
		}
	else
		{
			if ( m_linkPos == NULL )
				return;

			pLink = m_linkList.GetNext( m_linkPos );
		}

	//
	// Use the requested font
  CFont *pOrigFont = m_pDC->SelectObject( pFont );
	RECT rect;

	rDrawStr.TrimLeft();
	rDrawStr.TrimRight();

	CSize sz = m_pDC->GetTextExtent( rDrawStr );
		
	rect.left = m_nNextDrawX + nXExtra;
	rect.top = m_nNextDrawY + nYExtra;
	rect.right = rect.left + sz.cx;
	rect.bottom = rect.top + sz.cy;

	if ( m_bFirstPaint == TRUE )
		{

			pLink->Create
				( NULL, rDrawStr, WS_CHILD, rect, this, m_linkList.GetCount()+9000 );

			pLink->SetLinkTarget( rLinkStr );
			pLink->SetTextColour( 0x008f0000 ); // Text colour blue
			pLink->SetBackColour( 0x00ffffff ); // Text colour blue
			pLink->SetActiveTextColour( 0x00008f00 ); // Dark red
			pLink->SetTextPosition( HL_TEXTPOS_TOP, HL_TEXTALIGN_LEFT );
			pLink->SetFontName( "Arial" );
			pLink->SetFontSize( -14 );
			pLink->SetNoBitmap();
			pLink->SetFontAttributes( HL_FONTNOTBOLD, HL_FONTNOTITALIC );
			// pLink->SetFontToUse( pFont );
			pLink->AllowModsToFont();
			pLink->ShowWindow( SW_SHOW );
		}

	//
	// Set basic Y co-ordinate for next string (X stays the same for the moment)
	m_nNextDrawY = rect.bottom + DEFAULT_Y_PADDING;

	//
	// Set basic X co-ordinate for next invocation of DrawString
	m_nNextDrawX = DEFAULT_X_PADDING;

	//
	// Reset font
  m_pDC->SelectObject( pOrigFont );
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl::DrawString
// Draw the given string with the given font.
void PG_EntryViewCtrl::DrawString( CString &str, CFont *pFont, int nXExtra, int nYExtra )
{
	RECT rect;

	//
	// Use the requested font
  CFont *pOrigFont = m_pDC->SelectObject( pFont );

	//
	// Split the (possibly) multi-line string into lines
	// and draw them individually
	QStringList strList;
	strList.SetSplitString( str, '\n' );

	POSITION pos = strList.GetHeadPosition();
	while ( pos != NULL )
		{
			CString drawStr = strList.GetNext( pos );
			drawStr.TrimLeft();
			drawStr.TrimRight();
		  CSize sz = m_pDC->GetTextExtent( drawStr );

			rect.left = m_nNextDrawX + nXExtra;
			rect.top = m_nNextDrawY + nYExtra;

			rect.right = rect.left + sz.cx;
			rect.bottom = rect.top + sz.cy;

			//
			// Draw the text
			m_pDC->DrawText( drawStr, &rect, DT_NOCLIP|DT_LEFT );

			//
			// Set basic Y co-ordinate for next string (X stays the same for the moment)
			m_nNextDrawY = rect.bottom + DEFAULT_Y_PADDING;
		}

	//
	// Set basic X co-ordinate for next invocation of DrawString
	m_nNextDrawX = DEFAULT_X_PADDING;

	//
	// Reset font
  m_pDC->SelectObject( pOrigFont );
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryViewCtrl message handlers
void PG_EntryViewCtrl::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

  RECT rect;
  GetClientRect( &rect );

  CRgn rgn;
  rgn.CreateRectRgn( rect.left, rect.top, rect.right, rect.bottom );

  dc.SetBkColor( RGB( 0xff, 0xff, 0xff ) );
  dc.PaintRgn( &rgn );

  DrawEntry( static_cast<CDC *>( &dc ) );
	m_bFirstPaint = FALSE;
}

void PG_EntryViewCtrl::OnDestroy() 
{
	CWnd::OnDestroy();
	ClearEntry();
}
