// PG_IconMap.cpp: implementation of the PG_IconMap and
// PG_IconMapElement classes.
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PCPages32.h"
#include "PG_IconMap.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PG_IconMapElement::PG_IconMapElement( PtrList<QOC> &rOCList, int nIconID )
{
	POSITION pos = rOCList.GetHeadPosition();
	while ( pos != NULL )
		{
			QOC *pOC = rOCList.GetNext( pos );
			m_OCList.AddTail( pOC );
		}

	m_nIconID = nIconID;
}

PG_IconMapElement::~PG_IconMapElement()
{
}

//////////////////////////////////////////////////////////////////////
// PG_IconMapElement::QueryGetIconID
//
// Check if this icon mapping matches the given object class value.
// If so return the stored icon identifier.
BOOL PG_IconMapElement::QueryGetIconID( QOC *pOC, int &rIconID )
{
	if ( m_OCList.Find( pOC ) != NULL )
		{
			rIconID = m_nIconID;
			return TRUE;
		}

	return FALSE;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PG_IconMap::PG_IconMap()
{
	m_defaultID = -1;
}

PG_IconMap::~PG_IconMap()
{
	POSITION pos = m_iconList.GetHeadPosition();
	while ( pos != NULL )
		delete m_iconList.GetNext( pos );
}

//////////////////////////////////////////////////////////////////////
// PG_IconMap::Load
//
// Load the icon map from the registry.
BOOL PG_IconMap::Load()
{
	extern CPCPages32App theApp;

	HKEY hKey = theApp.OpenAppKey( "Icons" );
	if ( hKey == NULL )
		{
			theApp.ErrorBox( PGS_REGINIT_FAILED );
			return FALSE;
		}

	//
	// Get key/value information
	DWORD dwNumValues;
	DWORD dwMaxValueLen;
	DWORD dwMaxValueNameLen;
	
	LONG ok = RegQueryInfoKey
		(
			hKey,                // handle to key to query
			NULL,           // address of buffer for class string
			NULL,        // address of size of class string buffer
			NULL,       // reserved
			NULL,       // address of buffer for number of subkeys
			NULL,  // address of buffer for longest subkey name length
			NULL,  // address of buffer for longest class string length
			&dwNumValues,        // address of buffer for number of value entries
			&dwMaxValueNameLen,  // address of buffer for longest value name length
			&dwMaxValueLen,  // address of buffer for longest value data length
			NULL,           // address of buffer for security descriptor length
			NULL						// address of buffer for last write time
		);

	//
	// Create a buffer for loading
	BYTE pBuf[ 1024 ];
	
	//
	// Read the number of icons
	if ( theApp.QueryValue
				( hKey, "Icon Count", REG_DWORD, pBuf, &dwMaxValueLen ) == FALSE )
		{
			theApp.ErrorBox( PGS_REGINIT_FAILED );
			return FALSE;
		}

	int nIconCount = static_cast<int>( *pBuf );

	//
	// Read the icon mappings
	for ( int nIconID = 0; nIconID < nIconCount; nIconID++ )
		{
			char valName[16];
			sprintf( valName, "Icon%d", nIconID );

			dwMaxValueLen = 1024;
			if ( theApp.QueryValue
						( hKey, valName, REG_SZ, pBuf, &dwMaxValueLen ) == TRUE )
				{
					CString ocStr( reinterpret_cast<char *>( pBuf ) );

					//
					// Check for the default icon...
					if ( ocStr == "=default" )
						{
							m_defaultID = nIconID;
							continue;
						}

					//
					// ...otherwise read object class(es)
					QStringList ocStrList;
					ocStrList.SetSplitString( ocStr, ';' );
					PtrList<QOC> ocList;

					POSITION pos = ocStrList.GetHeadPosition();
					while ( pos != NULL )
						{
							ocStr = ocStrList.GetNext( pos );
							QOC *pOC = theApp.DS()->Table()->str2oc( ocStr );
							if ( pOC != NULL )
								ocList.AddTail( pOC );
#ifdef _DEBUG
							else
								{
									TRACE
										( "Unrecognised object class (%s) found in registry key \"Icon List\".\n",
											ocStr );
								}
#endif
						}

					if ( ocList.GetCount() > 0 )
						{
							PG_IconMapElement *pElem =
								new PG_IconMapElement( ocList, nIconID );

							m_iconList.AddTail( pElem );
						}
				}
		}

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// PG_IconMap::GetIconID
//
// Get the icon identifier corresponding to the given object class
// value. 
BOOL PG_IconMap::GetIconID( QOC *pOC, int &rIconID )
{
	POSITION pos = m_iconList.GetHeadPosition();
	while ( pos != NULL )	
		{
			PG_IconMapElement *pElem = m_iconList.GetNext( pos );
			if ( pElem->QueryGetIconID( pOC, rIconID ) == TRUE )
				return TRUE;
		}

	return FALSE;
}

//////////////////////////////////////////////////////////////////////
// PG_IconMap::GetIconID
//
// Get an icon identifier that maps to the object class value of
// the given entry. If no mathcing icon mapping found, return the
// default icon ID. 
BOOL PG_IconMap::GetIconID( QEntry *pEntry, int &rIconID )
{
	extern CPCPages32App theApp;
	PtrList<QValue> valList;

	QAttr *pAttr = pEntry->ObjectClass();
	pAttr->GetValues( valList );

	POSITION pos = valList.GetHeadPosition();
	while ( pos != NULL )
		{
			CString ocValStr;
			QValue *pVal = valList.GetNext( pos );
			pVal->Print( ocValStr );

			QOC *pOC = theApp.Oidtable()->str2oc( ocValStr );
			if ( pOC != NULL && GetIconID( pOC, rIconID ) == TRUE )
				return TRUE;
		}

	if ( m_defaultID >= 0 )
		{
			rIconID = m_defaultID;
			return TRUE;
		}

	return FALSE;
}

